;;; zenirc-history.el --- implementation of simple command history in zenirc

;; Copyright (C) 1995 Dan Weiskopf

;; Author: Dan Weiskopf <debaser@netcom.com>
;; Maintainer: Dan Weiskopf <debaser@netcom.com>
;; Keywords: extensions, zenirc, history
;; Created: 1995-06-28
;; Last modified: 1995-08-23

;; $Id: zenirc-history.el,v 1.10 1995/09/05 13:18:02 debaser Exp $
;; $Date: 1995/09/05 13:18:02 $

;; This program is free software; you can redistribute it and/or modify
;; it under the terms of the GNU General Public License as published by
;; the Free Software Foundation; either version 2, or (at your option)
;; any later version.
;;
;; This program is distributed in the hope that it will be useful,
;; but WITHOUT ANY WARRANTY; without even the implied warranty of
;; MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
;; GNU General Public License for more details.
;;
;; You should have received a copy of the GNU General Public License
;; along with this program; if not, you can either send email to this
;; program's maintainer or write to: The Free Software Foundation,
;; Inc.; 675 Massachusetts Avenue; Cambridge, MA 02139, USA.

;;; Commentary:
;;; About time.

;;; Code:

(require 'zenirc)
(require 'ring)

(defvar zenirc-history-size 25
  "*Number of elements to store in the zenirc command history ring.
This variable should only be set before the package is loaded (that is,
before the ring itself is initialized).  Otherwise, use zenirc-history-resize
to change the ring's size (this is, by default, hooked to the command
\"/history\".")

(defvar zenirc-history-ring (make-ring zenirc-history-size)
  "Ring used for storing zenirc's history list.")

(defvar zenirc-history-item 0
  "Current item in the history list.
This should never be set directly.")

(setq zenirc-send-line-hook '(zenirc-history-insert))

(define-key zenirc-mode-map "\M-p" 'zenirc-history-retrieve-previous)
(define-key zenirc-mode-map "\M-n" 'zenirc-history-retrieve-next)
(define-key zenirc-mode-map "\C-\M-p" 'zenirc-history-regex-retrieve-previous)
(define-key zenirc-mode-map "\C-\M-n" 'zenirc-history-regex-retrieve-next)

(zenirc-lang-store-catalog-entry
 'history "[info] The history ring has %s elements in it.\n" 'english)

(defvar zenirc-command-history-hook 'zenirc-command-history)


(defun zenirc-history-resize (&optional size)
  "Interactively resize the zenirc history ring.
All of the old values in the history ring will be preserved, unless the
ring is shrunk."
  (interactive "nNew size of zenirc history ring: ")
  (let ((count 0)
	(iters 0)
	(real-ring-size (ring-length zenirc-history-ring))
	(zenirc-new-history-ring (make-ring size)))
    (cond ((> size zenirc-history-size)	; ring grows
	   (progn
	     (if (>= real-ring-size zenirc-history-size)
		 (setq iters zenirc-history-size)
	       (setq iters real-ring-size))
	     (while (< count iters)
	       (ring-insert zenirc-new-history-ring
			    (ring-remove zenirc-history-ring count))
	       (setq count (1+ count)))))
	  ((< size zenirc-history-size)	; ring shrinks
	   (progn
	     (if (>= size real-ring-size)
		 (setq iters real-ring-size)
	       (setq iters size))
	     (while (< count iters)
	       (ring-insert zenirc-new-history-ring
			    (ring-remove zenirc-history-ring count))
	       (setq count (1+ count))))))
    (setq zenirc-history-ring zenirc-new-history-ring)
    (setq zenirc-history-size size)))

(defun zenirc-command-history (proc parsedcmd)
  "Interactively resize the zenirc history buffer.
With no argument, displays the current size of the history ring."
  (let ((count (string-to-number (cdr parsedcmd))))
    (if (eq count 0)
	(zenirc-message proc 'history zenirc-history-size)
      (zenirc-history-resize count))))

(defun zenirc-history-cleanup-command ()
  (goto-char zenirc-process-mark)
  (delete-region (point) (point-max)))

(defun zenirc-history-insert (begin end cmd)
  (ring-insert zenirc-history-ring cmd)
  (setq zenirc-history-item 0))

(defun zenirc-history-retrieve-previous ()
  (interactive)
  (zenirc-history-cleanup-command)
  (zenirc-insert-at-proc-mark
   (ring-ref zenirc-history-ring zenirc-history-item))
  (setq zenirc-history-item (1+ zenirc-history-item)))

(defun zenirc-history-retrieve-next ()
  (interactive)
  (zenirc-history-cleanup-command)
  (zenirc-insert-at-proc-mark
   (ring-ref zenirc-history-ring zenirc-history-item))
  (setq zenirc-history-item (1- zenirc-history-item)))

(defun zenirc-history-make-regex ()
  "Construct a regular expression of the irc command currently being entered."
  (let ((string (buffer-substring zenirc-process-mark (point-max))))
    (setq string (concat "^" (regexp-quote string) ".*"))))

(defun zenirc-history-regex-retrieve-previous ()
  (interactive)
  (let ((regex (zenirc-history-make-regex))
	(count 0))
    (if
	(catch 'match
	  (while (<= count zenirc-history-size)
	    (if (string-match regex
			      (ring-ref zenirc-history-ring count))
		(throw 'match count)
	      (setq count (1+ count)))))
	(progn
	  (zenirc-history-cleanup-command)
	  (zenirc-insert-at-proc-mark
	   (ring-ref zenirc-history-ring count))))))

(defun zenirc-history-regex-retrieve-next ()
  (interactive)
  (let ((regex (zenirc-history-make-regex))
	(count zenirc-history-size))
    (if
	(catch 'match
	  (while (>= count (- zenirc-history-size))
	    (if (string-match regex
			      (ring-ref zenirc-history-ring count))
		(throw 'match count)
	      (setq count (1- count)))))
	(progn
	  (zenirc-history-cleanup-command)
	  (zenirc-insert-at-proc-mark
	   (ring-ref zenirc-history-ring count))))))


(provide 'zenirc-history)

;;; zenirc-history.el ends here
